@@ -5,6 +5,8 @@ use std::ffi::OsStr;
 use std::ffi::OsString;
 use std::fs::Metadata;
 use std::fs::ReadDir;
+use std::hash::Hash;
+use std::hash::Hasher;
 use std::io;
 use std::mem;
 use std::ops::Deref;
@@ -16,9 +18,20 @@ use std::path::PathBuf;
 use super::error::MissingPrefixBufError;
 use super::error::MissingPrefixError;
 use super::error::ParentError;
-use super::normalize;
+use super::imp;
 use super::PathExt;
 
+fn cow_path_into_base_path(path: Cow<'_, Path>) -> Cow<'_, BasePath> {
+    debug_assert!(imp::is_base(&path));
+
+    match path {
+        Cow::Borrowed(path) => {
+            Cow::Borrowed(BasePath::from_inner(path.as_os_str()))
+        }
+        Cow::Owned(path) => Cow::Owned(BasePathBuf(path)),
+    }
+}
+
 /// A borrowed path that has a [prefix] on Windows.
 ///
 /// Note that comparison traits such as [`PartialEq`] will compare paths
@@ -31,17 +44,13 @@ use super::PathExt;
 /// panic if this path is missing a prefix on Windows. A safe `new_unchecked`
 /// method might be added later that can safely create invalid base paths.
 ///
-/// Although this type is annotated with `#[repr(transparent)]`, the inner
-/// representation is not stable. Transmuting between this type and any other
-/// causes immediate undefined behavior.
-///
 /// [prefix]: ::std::path::Prefix
 #[derive(Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
 #[repr(transparent)]
 pub struct BasePath(pub(super) OsStr);
 
 impl BasePath {
-    fn from_inner(path: &OsStr) -> &Self {
+    pub(super) fn from_inner(path: &OsStr) -> &Self {
         // SAFETY: This struct has a layout that makes this operation safe.
         unsafe { mem::transmute(path) }
     }
@@ -83,7 +92,7 @@ impl BasePath {
         match path {
             Cow::Borrowed(path) => Self::try_new(path)
                 .map(Cow::Borrowed)
-                .or_else(|_| normalize::to_base(path).map(Cow::Owned)),
+                .or_else(|_| imp::to_base(path).map(Cow::Owned)),
             Cow::Owned(path) => BasePathBuf::new(path).map(Cow::Owned),
         }
     }
@@ -119,7 +128,7 @@ impl BasePath {
         P: AsRef<Path> + ?Sized,
     {
         let path = path.as_ref();
-        if normalize::is_base(path) {
+        if imp::is_base(path) {
             Ok(Self::from_inner(path.as_os_str()))
         } else {
             Err(MissingPrefixError(()))
@@ -144,8 +153,8 @@ impl BasePath {
     #[inline]
     pub fn canonicalize(&self) -> io::Result<BasePathBuf> {
         self.as_path().canonicalize().map(|base| {
-            debug_assert!(normalize::is_base(&base));
-            BasePathBuf(base.into_os_string())
+            debug_assert!(imp::is_base(&base));
+            BasePathBuf(base)
         })
     }
 
@@ -172,6 +181,12 @@ impl BasePath {
         self.as_path().exists()
     }
 
+    /// Equivalent to [`PathExt::expand`].
+    #[inline]
+    pub fn expand(&self) -> io::Result<Cow<'_, Self>> {
+        self.as_path().expand().map(cow_path_into_base_path)
+    }
+
     /// Equivalent to [`Path::extension`].
     #[inline]
     #[must_use]
@@ -228,6 +243,13 @@ impl BasePath {
         self.as_path().is_relative()
     }
 
+    /// Equivalent to [`Path::is_symlink`].
+    #[inline]
+    #[must_use]
+    pub fn is_symlink(&self) -> bool {
+        self.as_path().is_symlink()
+    }
+
     /// An improved version of [`Path::join`] that handles more edge cases.
     ///
     /// For example, on Windows, leading `.` and `..` components of `path` will
@@ -385,6 +407,12 @@ impl BasePath {
         self.as_path().read_link()
     }
 
+    /// Equivalent to [`PathExt::shorten`].
+    #[inline]
+    pub fn shorten(&self) -> io::Result<Cow<'_, Self>> {
+        self.as_path().shorten().map(cow_path_into_base_path)
+    }
+
     /// Equivalent to [`Path::starts_with`].
     #[inline]
     #[must_use]
@@ -400,6 +428,12 @@ impl BasePath {
     pub fn symlink_metadata(&self) -> io::Result<Metadata> {
         self.as_path().symlink_metadata()
     }
+
+    /// Equivalent to [`Path::try_exists`].
+    #[inline]
+    pub fn try_exists(&self) -> io::Result<bool> {
+        self.as_path().try_exists()
+    }
 }
 
 impl AsRef<OsStr> for BasePath {
@@ -463,7 +497,7 @@ impl ToOwned for BasePath {
 
     #[inline]
     fn to_owned(&self) -> Self::Owned {
-        BasePathBuf(self.0.to_owned())
+        BasePathBuf(self.0.to_owned().into())
     }
 }
 
@@ -472,8 +506,8 @@ impl ToOwned for BasePath {
 /// For more information, see [`BasePath`].
 ///
 /// [prefix]: ::std::path::Prefix
-#[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-pub struct BasePathBuf(pub(super) OsString);
+#[derive(Clone, Debug)]
+pub struct BasePathBuf(pub(super) PathBuf);
 
 impl BasePathBuf {
     /// Equivalent to [`BasePath::new`] but returns an owned path.
@@ -500,7 +534,7 @@ impl BasePathBuf {
     where
         P: Into<PathBuf>,
     {
-        Self::try_new(path).or_else(|x| normalize::to_base(&x.0))
+        Self::try_new(path).or_else(|x| imp::to_base(&x.0))
     }
 
     /// Equivalent to [`BasePath::try_new`] but returns an owned path.
@@ -528,8 +562,8 @@ impl BasePathBuf {
         P: Into<PathBuf>,
     {
         let path = path.into();
-        if normalize::is_base(&path) {
-            Ok(Self(path.into_os_string()))
+        if imp::is_base(&path) {
+            Ok(Self(path))
         } else {
             Err(MissingPrefixBufError(path))
         }
@@ -539,14 +573,14 @@ impl BasePathBuf {
     #[inline]
     #[must_use]
     pub fn into_os_string(self) -> OsString {
-        self.0
+        self.0.into_os_string()
     }
 
     /// Returns the wrapped path.
     #[inline]
     #[must_use]
     pub fn into_path_buf(self) -> PathBuf {
-        self.0.into()
+        self.0
     }
 
     /// Equivalent to [`BasePath::parent`] but modifies `self` in place.
@@ -574,13 +608,6 @@ impl BasePathBuf {
         self.check_parent().map(|()| self.pop_unchecked())
     }
 
-    pub(super) fn replace_with<F>(&mut self, replace_fn: F)
-    where
-        F: FnOnce(PathBuf) -> PathBuf,
-    {
-        self.0 = replace_fn(mem::take(&mut self.0).into()).into_os_string();
-    }
-
     /// Equivalent to [`PathBuf::pop`].
     ///
     /// It is usually better to use [`pop`].
@@ -602,13 +629,7 @@ impl BasePathBuf {
     /// [`pop`]: Self::pop
     #[inline]
     pub fn pop_unchecked(&mut self) -> bool {
-        // This value is never used.
-        let mut result = false;
-        self.replace_with(|mut base| {
-            result = base.pop();
-            base
-        });
-        result
+        self.0.pop()
     }
 
     /// Equivalent to [`BasePath::join`] but modifies `self` in place.
@@ -631,21 +652,21 @@ impl BasePathBuf {
     where
         P: AsRef<Path>,
     {
-        normalize::push(self, path.as_ref());
+        imp::push(self, path.as_ref());
     }
 }
 
 impl AsRef<OsStr> for BasePathBuf {
     #[inline]
     fn as_ref(&self) -> &OsStr {
-        &self.0
+        self.as_os_str()
     }
 }
 
 impl AsRef<Path> for BasePathBuf {
     #[inline]
     fn as_ref(&self) -> &Path {
-        self.as_path()
+        &self.0
     }
 }
 
@@ -668,10 +689,12 @@ impl Deref for BasePathBuf {
 
     #[inline]
     fn deref(&self) -> &BasePath {
-        BasePath::from_inner(&self.0)
+        BasePath::from_inner(self.0.as_os_str())
     }
 }
 
+impl Eq for BasePathBuf {}
+
 impl From<BasePathBuf> for Cow<'_, BasePath> {
     #[inline]
     fn from(value: BasePathBuf) -> Self {
@@ -682,14 +705,46 @@ impl From<BasePathBuf> for Cow<'_, BasePath> {
 impl From<BasePathBuf> for OsString {
     #[inline]
     fn from(value: BasePathBuf) -> Self {
-        value.0
+        value.into_os_string()
     }
 }
 
 impl From<BasePathBuf> for PathBuf {
     #[inline]
     fn from(value: BasePathBuf) -> Self {
-        value.into_path_buf()
+        value.0
+    }
+}
+
+impl Hash for BasePathBuf {
+    #[inline]
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        (**self).hash(state);
+    }
+}
+
+impl Ord for BasePathBuf {
+    #[inline]
+    fn cmp(&self, other: &Self) -> Ordering {
+        (**self).cmp(&**other)
+    }
+}
+
+impl PartialEq for BasePathBuf {
+    #[inline]
+    fn eq(&self, other: &Self) -> bool {
+        **self == **other
+    }
+}
+
+impl PartialOrd for BasePathBuf {
+    #[expect(clippy::non_canonical_partial_ord_impl)]
+    #[inline]
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        (**self).partial_cmp(&**other)
     }
 }
 
@@ -750,7 +805,7 @@ mod serde {
         where
             D: Deserializer<'de>,
         {
-            OsString::deserialize(deserializer).map(Self)
+            OsString::deserialize(deserializer).map(|x| Self(x.into()))
         }
     }
 
@@ -770,7 +825,8 @@ mod serde {
         where
             S: Serializer,
         {
-            serializer.serialize_newtype_struct("BasePathBuf", &self.0)
+            serializer
+                .serialize_newtype_struct("BasePathBuf", self.as_os_str())
         }
     }
 }
